-- EasyColorPicker v2.0
-----------------------
local debug = false
local function print(...)
	if ( debug ) then _G.print(...) end
end

local ColorPickerFrame, OpacitySliderFrame, ColorSwatch = _G["ColorPickerFrame"], _G["OpacitySliderFrame"], _G["ColorSwatch"]

local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
	tile = true, edgeSize = 1, tileSize = 5,
}

-- allow draggability
local anchor = CreateFrame("Frame", "ColorPickerFrameAnchor", ColorPickerFrame)
anchor:SetSize(120, 24)
anchor:SetPoint("Top", ColorPickerFrame, "Top", 0, 4)
anchor:EnableMouse(true)
anchor:SetMovable(true)

anchor:SetScript("OnMouseDown", function(self)
		if self:IsMovable() then
			ColorPickerFrame:StartMoving()
		end
	end)

anchor:SetScript("OnMouseUp", function(self)
		if self:IsMovable() then
			ColorPickerFrame:StopMovingOrSizing()
		end
	end)

-- make it lockable
-- RELOAD BUTTON
if ( debug ) then
	local button = CreateFrame("Button", "ReloadUIButton", UIParent, "UIPanelButtonTemplate")
	button:SetText("Reload UI!")
	button:SetSize(200, 56)
	button:SetPoint("Center", UIParent, "Center", -500, 0)

	button:SetScript("PostClick", function(...)
		ReloadUI()
	end)
	
end

----------------------------------------------------------
-- error template

local errorFrame = CreateFrame("Frame", "ColorPickerFrameErrorFrame", ColorPickerFrame)
errorFrame:SetSize(100, 44)
errorFrame:SetPoint("Bottomright", ColorPickerFrame, "Topright", -10, 0)

errorFrame:SetBackdrop(backdrop)
errorFrame:SetBackdropColor(0, 0, 0, 0.5)
errorFrame:SetBackdropBorderColor(0.3, 0.3, 0.3, 0.8)

errorFrame.errorString = errorFrame:CreateFontString(nil, "Artwork", "GameFontNormalSmall")
errorFrame.errorString:SetPoint("Topleft", errorFrame, "Topleft", 0, 0)
errorFrame.errorString:SetPoint("Bottomright", errorFrame, "Bottomright", 0, 0)
errorFrame.errorString:SetTextColor(1, 1, 1)

errorFrame:Hide()

function errorFrame:Assign(formatString, ...)
	self.errorString:SetFormattedText(formatString, ...)
	self:Show()
end

function errorFrame:Clear()
	self:Hide()
end
----------------------------------------------------------

local function roundToDecimal(decimal, integer, returnAsNumber)
	local formatString = "%." .. decimal .. "f"
	local roundedString = string.format(formatString, integer)
	return returnAsNumber and tonumber(roundedValue) or roundedString
end

local function buildHexString(...)
	local hexString = ""
	for i=1,3 do 
		if select(i, ...) < 16 then
			hexString = string.format("%s%s%s", hexString, 0, format("%X", select(i, ...)))
		else
			hexString = string.format("%s%s", hexString, format("%X", select(i, ...)))
		end
	end
	return hexString
end

local function getHexString()
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local hexString = buildHexString(r*255, g*255, b*255)

	return hexString
end

-- standard calls for scripts
local function EditBox_OnEnterPressed(frame)
	frame.oldValue = frame:GetText()
	PlaySound("igMainMenuOptionCheckBoxOn")
	frame:HighlightText(0, 0)
	frame:ClearFocus()
end

local function EditBox_OnEscapePressed(frame)
	-- need to revert changes here!
	-- restore old editbox text in case it got deleted
	frame:SetText(frame.oldValue)
	ColorPickerFrame:SetColorRGB(ColorSwatch_New.r, ColorSwatch_New.g, ColorSwatch_New.b)
	if ( frame:GetName() == "ColorPickerFrameEditBoxOpacity" ) then
		OpacitySliderFrame:SetValue(1 - ColorPickerFrameEditBoxOpacity.oldValue)
	end
	frame:HighlightText(0, 0)
	frame:ClearFocus()
end

local function EditBox_OnEnter(frame)
	frame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
end

local function EditBox_OnLeave(frame)
	frame:SetBackdropBorderColor(0.3, 0.3, 0.3, 0.8)
end

local function EditBox_OnEditFocusGained(frame)
	-- want to store original value of the editbox
	frame.oldValue = frame:GetText()
	-- in this case we never want to de-highlight the currently focused editbox
	-- even if we're mouseovering another editbox
	frame:SetScript("OnLeave", nil)
	EditBox_OnEnter(frame)
end

local function EditBox_OnEditFocusLost(frame)
	errorFrame:Clear()
	--frame:SetText(frame.oldValue)
	frame:SetScript("OnLeave", EditBox_OnLeave)
	-- ignore OnTextChanged script if not focused!
	frame:SetScript("OnTextChanged", nil)
	EditBox_OnLeave(frame)
end

do
	----------------------------------------------

	--XXX OnEnterPressed necessary, if on the fly adjustment are done on OnChar?!?

	-- red, green, blue editboxes:
	local function RGB_OnEnterPressed(self)
		EditBox_OnEnterPressed(self)
		r, g, b = ColorPickerFrameEditBoxRed:GetNumber()/255, ColorPickerFrameEditBoxGreen:GetNumber()/255, ColorPickerFrameEditBoxBlue:GetNumber()/255
		ColorSwatch_New.r, ColorSwatch_New.g, ColorSwatch_New.b = r, g, b
		self:SetNumber(self:GetNumber())
		if ( self:GetNumber() > 255 ) then self:SetText(255) end
		if ( self:GetText() == "" ) then self:SetText(0) end
	end

	local function RGB_OnTextChanged(self, userInput)
		local editbox = "ColorPickerFrameEditBox"
		local r, g, b = _G[editbox.."Red"]:GetNumber()/255, _G[editbox.."Green"]:GetNumber()/255, _G[editbox.."Blue"]:GetNumber()/255 
		self.disableUpdate = true

		if ( self:GetNumber() > 255 ) then
			local errorString = "You may not use\nnumbers > |cFFFF0000%s|r\n in this editbox!"
			errorFrame:Assign(errorString, 255)
		end

		ColorPickerFrame:SetColorRGB(r, g, b)
	end

	-- hex editbox:
	local function Hex_OnEnterPressed(self)
		EditBox_OnEnterPressed(self)
		local text = self:GetText()

		for i = text:len(), 5 do
			text = text .. 0
		end

		ColorSwatch_New.r, ColorSwatch_New.g, ColorSwatch_New.b = ColorPickerFrame:GetColorRGB()
		self:SetText(text)
	end

	local function Hex_OnTextChanged(self, userInput)
		local text = self:GetText()
		if ( text:find("[%X]") ) then
			-- remove any non-hexadecimal letters
			local errorString = "You may not use\n |cFFFF0000%s|r \nin this editbox!"
			errorFrame:Assign(errorString, text:gsub("[%x]", ""):upper())
			self:SetText(text:gsub("[%X]", ""))
		else
			self.disableUpdate = true

			if ( text:find("[a-f]") ) then
				-- make everything uppercase
				self:SetText(text:upper())
			end

			for i = text:len(), 5 do
				text = text .. 0
			end

			local r, g, b = tonumber(text:sub(1,2), 16)/255, tonumber(text:sub(3,4), 16)/255, tonumber(text:sub(5,6), 16)/255 
			ColorPickerFrame:SetColorRGB(r, g, b)
		end
	end

	-- opacity editbox:
	local function Opacity_OnEnterPressed(self)
		EditBox_OnEnterPressed(self)
		local alpha = tonumber(self:GetText())
		--local alpha = tonumber(self:GetText():format("%.3f"))

		alpha = ( alpha and  alpha > 1 and 1 or alpha ) or 0
		OpacitySliderFrame:SetValue(1 - alpha)
		self:SetText(roundToDecimal(3, alpha))
	end

	local function Opacity_OnTextChanged(self, userInput)
		local text, number = self:GetText(), self:GetNumber()
		local match = text:match("%d%.%d*")

		if ( text:find("[^%d%.]") ) then
			-- ignroe anything which isn't [0-9%.]
			local errorString = "You may not use\n |cFFFF0000%s|r \nin this editbox!"
			errorFrame:Assign(errorString, text:gsub("[%d%.]", ""):upper())

			self:SetText(text:gsub("[^%d%.]", ""))
		else
			self.disableUpdate = true
			local alpha

			if ( text:match("%d%.%d*") ) then
				print("1 - " .. tostring(text:match("%d%.%d*")))
				self:SetText(text:match("%d%.%d*"))
				alpha = roundToDecimal(3, text:match("%d%.%d*"))
			elseif ( text:match("%.%d*") ) then
				print("2 - " .. tostring(text:match("%.%d*")))
				self:SetText(text:match("%.%d*"))
				alpha = roundToDecimal(3, tonumber(0 .. text:match("%.%d*")))
			elseif ( text:match("%d*") )then
				print("3 - " .. text:match("%d*"))
				alpha = number >= 1 and 1 or 0
			end

			print(tostring(alpha))
			OpacitySliderFrame:SetValue(1 - alpha)
		end
	end

	----------------------------------------------------------------------

	local r, g, b = ColorPickerFrame:GetColorRGB()
	local hex, alpha = getHexString(), OpacitySliderFrame:GetValue()

	local order = {
		[1] = "Red",
		[2] = "Green",
		[3] = "Blue",
		[4] = "Hex",
		[5] = "Opacity",
		--[6] = "Hue",
		--[7] = "Saturation",
		--[8] = "Lightness",
	}

	local editboxValues = {
	["Red"] 	= { text = "R", value = r*255,  parent = ColorPickerFrame,   numeric = true,  maxLetters = 3, width = 46, point = { "Bottomleft", ColorPickerFrame, "Bottomleft", 28, 38 },
					OnEnterPressed = RGB_OnEnterPressed,	 OnTextChanged = RGB_OnTextChanged		},

	["Green"] 	= { text = "G", value = g*255,  parent = ColorPickerFrame,   numeric = true,  maxLetters = 3, width = 46, point = { "Left",		  "Red",			"Right",	  18, 0	 },
					OnEnterPressed = RGB_OnEnterPressed,	 OnTextChanged = RGB_OnTextChanged		},

	["Blue"]	= { text = "B", value = b*255,  parent = ColorPickerFrame,   numeric = true,  maxLetters = 3, width = 46, point = { "Left",		  "Green",			"Right",	  18, 0	 },
					OnEnterPressed = RGB_OnEnterPressed,	 OnTextChanged = RGB_OnTextChanged		},

	["Hex"]		= { text = "#", value = hex,	parent = ColorPickerFrame,   numeric = false, maxLetters = 6, width = 64, point = { "Left",		  "Blue",			"Right", 	  18, 0	 },
					OnEnterPressed = Hex_OnEnterPressed,	 OnTextChanged = Hex_OnTextChanged		},

	["Opacity"]	= { text = "A", value = alpha,  parent = OpacitySliderFrame, numeric = false, maxLetters = 5, width = 56, point = { "Left",		  "Hex",			"Right", 	  17, 0	 },	
					OnEnterPressed = Opacity_OnEnterPressed, OnTextChanged = Opacity_OnTextChanged	},
	--[[
	["Hue"] 	= { text = "H", value = r*255,	parent = ColorPickerFrame,	 numeric = true,  maxLetters = 3, width = 40, point = {	"Topleft",	  "Red",			"Bottomleft", 0, -4  }, 
					text2 = "°"},

	["Saturation"] = { text = "H", value = g*255,	parent = ColorPickerFrame,	 numeric = true,  maxLetters = 3, width = 40, point = {	"Left",		  "Hue",		"Left",		  68, 0  },
					text2 = "%" },
	
	["Lightness"] = { text = "V", value = b*255,	parent = ColorPickerFrame,	 numeric = true,  maxLetters = 3, width = 40, point = {	"Left",		  "Saturation",	"Left",		  68, 0  },
					text2 = "%" },
	]]
	}

	-------------------------------------------------------------------------------

	local function setupColorPicker(self)
		ColorPickerFrame:SetHeight(224)
		ColorPickerFrame:EnableKeyboard(false)

		ColorPickerFrame:HookScript("OnShow", function(self)
			if ( self.hasOpacity ) then
				self:SetWidth(374)
			end
		end)

		for i=1,#order do 
			local k = order[i] 
			local v = editboxValues[k]

			local editbox = CreateFrame("EditBox", "ColorPickerFrameEditBox"..k, v.parent)

			editbox:SetAutoFocus(false)
			editbox:SetFontObject("GameFontNormal")
			editbox:SetTextColor(1, 1, 1)
			editbox:SetJustifyH("CENTER")
			editbox:EnableMouse(true)
			editbox:SetBackdrop(backdrop)
			editbox:SetBackdropColor(0, 0, 0, 0.5)
			editbox:SetBackdropBorderColor(0.3, 0.3, 0.3, 0.8)
			------------------------------------------------------
			editbox:SetText(v.value)

			-- script handler
			editbox:SetScript("OnEnter", EditBox_OnEnter)
			editbox:SetScript("OnLeave", EditBox_OnLeave)
			editbox:SetScript("OnEditFocusGained", function(self) EditBox_OnEditFocusGained(self) self:SetScript("OnTextChanged", v.OnTextChanged) end)
			editbox:SetScript("OnEditFocusLost", function(self) EditBox_OnEditFocusLost(self) if (v.OnEnterPressed) then v.OnEnterPressed(self) end end)
			editbox:SetScript("OnEnterPressed", v.OnEnterPressed and v.OnEnterPressed or EditBox_OnEnterPressed)
			editbox:SetScript("OnEscapePressed", EditBox_OnEscapePressed)

			local relativeTo = type(v.point[2]) == "string" and _G["ColorPickerFrameEditBox"..v.point[2]] or v.point[2]
			editbox:SetPoint(v.point[1], relativeTo, v.point[3], v.point[4], v.point[5])
			editbox:SetNumeric(v.numeric)
			editbox:SetMaxLetters(v.maxLetters)
			editbox:SetSize(v.width, 17)

			local text = editbox:CreateFontString(nil, "Overlay", "GameFontNormal")
			text:SetJustifyH("LEFT")
			text:SetTextColor(1, 1, 1)
			text:SetPoint("Right", editbox, "Left", 0, 0)
			text:SetText(v.text)

			if ( v.text2 ) then
				local text2 = editbox:CreateFontString(nil, "Overlay", "GameFontNormal")
				text2:SetJustifyH("LEFT")
				text2:SetTextColor(1, 1, 1)
				text2:SetPoint("Left", editbox, "Right", 0, 0)
				text2:SetText(v.text2)
			end
		end

		-- reanchor ColorSwatchFrame
		ColorSwatch:ClearAllPoints()

		-- reanchor OpacitySliderFrame
		OpacitySliderFrame:ClearAllPoints()
		OpacitySliderFrame:SetPoint("Topleft", "ColorPickerFrame", "Topleft", 310, -28)
		OpacitySliderFrame:SetValueStep(0.001)
		OpacitySliderFrame:SetObeyStepOnDrag(true)

		-- ColorSwatch
		local backdrop = {
			bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
			edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
			tile = true, edgeSize = 2, tileSize = 5,
		}

		local colorSwatch = CreateFrame("Frame", "ColorSwatch", ColorPickerFrame)
		colorSwatch:SetSize(60, 70)
		colorSwatch:SetPoint("Topleft", ColorPickerFrame, "Topleft", 222, -56)
		colorSwatch:SetBackdrop(backdrop)	
		colorSwatch:SetBackdropBorderColor(0, 0, 0)
	
		colorSwatch.texture_new = colorSwatch:CreateTexture("ColorSwatch_New", "Overlay")
		colorSwatch.texture_new:SetPoint("Topleft", colorSwatch, "Topleft", 2, -2)
		colorSwatch.texture_new:SetPoint("Topright", colorSwatch, "Topright", -2, -2)
		colorSwatch.texture_new:SetHeight(35)
		colorSwatch.texture_new:SetTexture(1, 0, 1)

		colorSwatch.texture_current = colorSwatch:CreateTexture("ColorSwatch_Current", "Overlay")
		colorSwatch.texture_current:SetPoint("Bottomleft", colorSwatch, "Bottomleft", 2, 2)
		colorSwatch.texture_current:SetPoint("Bottomright", colorSwatch, "Bottomright", -2, 2)
		colorSwatch.texture_current:SetHeight(31)

		-- title top:
		colorSwatch.title1 = colorSwatch:CreateFontString(nil, "Overlay")
		colorSwatch.title1:SetFont("Fonts\\FRIZQT__.TTF", 13)
		colorSwatch.title1:SetPoint("Bottom", colorSwatch, "Top", 0, 1)
		--colorSwatch.font1:SetTextColor(0, 0, 0)
		colorSwatch.title1:SetText("New")
		
		-- title bottom:
		colorSwatch.title2 = colorSwatch:CreateFontString(nil, "Overlay")
		colorSwatch.title2:SetFont("Fonts\\FRIZQT__.TTF", 13)
		colorSwatch.title2:SetPoint("Top", colorSwatch, "Bottom", 0, -1)
		--colorSwatch.font2:SetTextColor(0, 0, 0)
		colorSwatch.title2:SetText("Current")

		setupColorPicker = nil
	end
	setupColorPicker()

end

-- hook OnColorSelect
ColorPickerFrame:HookScript("OnColorSelect", function(self, r, g, b)
	print("OnColorSelect")
	local hex = getHexString()
	local editboxes = {
		ColorPickerFrameEditBoxRed	 = r*255,
		ColorPickerFrameEditBoxGreen = g*255,
		ColorPickerFrameEditBoxBlue  = b*255,
		ColorPickerFrameEditBoxHex	 = hex,
		--ColorPickerFrameEditBoxHue = hue,
		--ColorPickerFrameEditBoxSaturation = saturation,
		--ColorPickerFrameEditBoxValue = value,
	}

	for k,v in pairs(editboxes) do
		k = _G[k]
		if not k.disableUpdate then
			if ( type(v) == "number" ) then 
				k:SetNumber(v) 
			else
				k:SetText(v)
			end
		end
		k.disableUpdate = false
	end

	ColorSwatch_New:SetTexture(r, g, b, 1)
end)

ColorPickerFrame:HookScript("OnShow", function(self)
	local ColorSwatch_New = _G["ColorSwatch_New"]
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local hex, alpha = getHexString(), OpacitySliderFrame:GetValue()

	print(r*255, g*255, b*255)
	-- current set values:
	ColorSwatch_New.r, ColorSwatch_New.g, ColorSwatch_New.b, ColorSwatch_New.a = r, g, b, alpha
	ColorSwatch_New:SetTexture(r, g, b, 1)
	ColorSwatch_Current:SetTexture(r, g, b, 1)
	ColorPickerFrameEditBoxOpacity:SetText(roundToDecimal(3, 1 - alpha))
end)

OpacitySliderFrame:HookScript("OnValueChanged", function(self, value)
	local editbox = _G["ColorPickerFrameEditBoxOpacity"]
	print("OnValueChanged: " .. value)
	if not editbox.disableUpdate then
		local value = 1 - value
		editbox:SetText(roundToDecimal(3, value))
	end
	editbox.disableUpdate = false
end)
